/*
 * @seisHeaderTable.java Version 1.1 03/04/2015
 *
 * Copyright (c) 2015 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package seismic.gui;

import java.awt.*;
import java.lang.*;
import javax.swing.*;
import javax.swing.table.*;

import gui.guiTable;

import seismic.seisStruct;

/** Class seisHeaderTable
 *  <p> This Class will display the IQSTRAT Data Well List Table.
 *
 *  @version 1.1 03/04/2015
 *  @author  John R. Victorine
 */

public class seisHeaderTable
{
  private int      iRows     = 0;
  private int      iColumns  = 6;
  private String   sColumn[] = { "Network",
                                 "Station",
                                 "Location",
                                 "Channel",
                                 "Start Date",
                                 "End Date" };
  private int      iColLength[] = { 10, 10, 10, 10, 20, 20 };

  private Object   oData[][] = null;
  private guiTable pTable    = null;

  private int      iData     = 0;
  private String   sData[][] = null;

  private int      iChange   = 0;

  /** Constructor seisHeaderTable()
   *  <p> This is the Constructor for this class.
   */

  public seisHeaderTable()
  {
    pTable = new guiTable(0, iColumns, sColumn);

    resetColumnWidths();
    pTable.showHorScroll(true);
  }

  /** Constructor seisHeaderTable( seisStruct st )
   *  <p> This is the Constructor for this class.
   * @param st  = seismic data structure
   */

  public seisHeaderTable( seisStruct st )
  {
    if (st != null)
    {
      sData = addData(st, iData, sData);
      if (sData != null)
        iData = sData.length;
    }

    populateList();

    if (iRows > 0)
      pTable = new guiTable(0, iRows, iColumns, sColumn, oData);
    else
      pTable = new guiTable(0, iColumns, sColumn);

    resetColumnWidths();
    pTable.showHorScroll(true);

    if (iChange == 0)
      iRows = 0;

    if (iRows > 0)
      pTable.setSelectedRow(0);
  }

  /** Method resetColumnWidths()
   *  <p> This method will reset the column widths to a specific size.
   */

  private void resetColumnWidths()
  {
    for (int i=0; i<iColumns; i++)
    {
      pTable.resizeColumn(i, iColLength[i]);
    }
  }

  /** Method close()
   * <p> This method will set global variables to null in order to force
   *     Java to deallocate memory
   */

  public void close()
  {
    sColumn    = null;
    iColLength = null;
    sData      = null;
    oData      = null;
    pTable     = null;
  }

  /** Method populateList()
   *  <p> This method will populate the IQSTRAT List data.
   */

  private void populateList()
  {
	int i, j;
    int iCount = 0;

    iChange = 0;
    iRows   = 1;

    oData = new String[1][iColumns];
    for (i=0; i<iColumns; i++)
    {
      oData[0][i] = new String("");
    }

    if (sData != null)
    {
      if (iData > 0)
      {
        iChange = 1;
        oData   = new Object[iData][iColumns];
        iCount  = 0;

        for (i=0; i<iData; i++)
        {
          for (j=0; j<iColumns; j++)
          {
            oData[iCount][j]  = new String( sData[i][j] );
          }
          iCount++;
        }

        iRows = iCount;
      }
    }
  }

  /* ===================================================================== *
   * -------------------------- SET METHODS ------------------------------ *
   * ===================================================================== */

  /** Method setData( seisStruct st )
   *  <p> This method will reload the Region List after adding, deleting from
   *      the formation name list.
   * @param st  = The IQSTRAT Data Well List Structure
   */

  public void setData( seisStruct st )
  {
    if (st != null)
    {
      sData = addData(st, iData, sData);
      if (sData != null)
        iData = sData.length;
    }

    populateList();

    pTable.reload( iRows, oData );

    if (iChange == 0)
      iRows = 0;
  }

  /** Method addData()
   *  <p> This method will add the file identifier data into list.
   *  @param st    = Seismic Data Structure
   *  @param sData = Array of Seismic Data File Identifier Data
   *  @param sData = Array of Seismic Data File Identifier Data
   */

  public String[][] addData( seisStruct st, int iData, String sData[][] )
  {
    int i, j;
    String stemp[][] = null;
    int    iCount    = 0;

    if (st != null)
    {
      iData++;
      stemp = new String[iData][iColumns];

      for (i=0; i<iData; i++)
      {
        for (j=0; j<iColumns; j++)
        {
          stemp[iCount][j] = new String("");
        }
      }

      if (iData > 1)
      {
        for (i=0; i<iData-1; i++)
        {
          for (j=0; j<iColumns; j++)
          {
            stemp[iCount][j] = new String( sData[i][j] );
          }
          iCount++;
        }
      }

      stemp[iCount][0] = new String(st.network);  // Network Code
      stemp[iCount][1] = new String(st.station);  // Station Identifier
      stemp[iCount][2] = new String(st.location); // Location Identifier
      stemp[iCount][3] = new String(st.channel);  // Channel Identifier

      stemp[iCount][4] = new String(st.tsStart.toString());  // Channel Identifier
      stemp[iCount][5] = new String(st.tsEnd.toString());  // Channel Identifier

      iCount++;

      sData = new String[iCount][iColumns];

      for (i=0; i<iCount; i++)
      {
        for (j=0; j<iColumns; j++)
        {
          sData[i][j] = new String( stemp[i][j] );
        }
      }

      stemp = null;
    }

    return (sData);
  }

  /** Method clear()
   *  <p> This method will clear the contents of the table.
   */

  public void clear()
  {
    iData = 0;
    sData = null;

    populateList();

    pTable.reload( iRows, oData );

    if (iChange == 0)
      iRows = 0;
  }

  /* ===================================================================== *
   * -------------------------- GET METHODS ------------------------------ *
   * ===================================================================== */

  /** Method getCount()
   *  <p> This method will return the total rows in the table.
   *  @return Total Number of Rows.
   */

  public int getCount() { return (iRows); }

  /** Method getRow()
   *  <p> This method will return the selected data id in the table.
   *  @return selected data id.
   */

  public int getRow() { return (pTable.getSelectedRow()); }

  /** Method getScrollPane()
   *  <p> This method will return the scroll pane.
   *  @return jScrollPane - Scroll Pane containing the Table.
   */

  public JScrollPane getScrollPane() { return (pTable.getScrollPane()); }
}

/*
 *  @version 1.1 03/04/2015
 *  @author  John Victorine
 */
