/*
 * @seisSmoothingPanel.java Version 1.1 03/24/2015
 *
 * Copyright (c) 2015 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package seismic.gui;

import java.util.Observable;
import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import java.sql.Timestamp;
import java.sql.Date;
import java.text.SimpleDateFormat;
import java.text.ParseException;

import cmn.cmnStruct;               // Global Common Data Structure
import seismic.seisStruct;          // Seismic Data Structure
import seismic.gui.seisPlotFrame;   // Seismic Plot Frame

import seismic.seisPlotXYStruct;    // Plot XY Plot Data Structure
import seismic.gui.seisPlotXYFrame; // Plot XY Plot Frame
import plot.plotSymbolStruct;       // Plot Symbol Data Structure
import plot.plotSymbol;             // Plot Symbol Class

import util.utilWaitAnimation;      // Wait Animation Dialog
import util.SwingWorker;            // Swing Worker

/** Class seisSmoothingPanel
 *  <p> This Class allows the user to select from a set of simple smoothing
 *      functions to remove the noise on the seismic data. As well as the
 *      ability to create a waveform from the data using the 1st 100 points.
 *
 *  @version 1.1 03/24/2015
 *  @author  John R. Victorine
 */

public class seisSmoothingPanel extends JPanel implements ActionListener
{
  private static final int _SEISMIC_AVG = seismic.seisTracksStruct._SEISMIC_AVG;
  private static final int _SEISMIC_MAG = seismic.seisTracksStruct._SEISMIC_MAG;

  private static final int SCALE[]   = seismic.seisTracksStruct.SCALE;

  // Input Variables

  private cmnStruct        stCMN     = null; // Common Data Structure
  private Observable       notifier  = null; // Observables Object

  // Global Variables

  private seisStruct       stData    = null; // Raw Seismic Data
  private seisStruct       stAvg     = null; // Average Seismic Data
  private seisStruct       stMag     = null; // Envelope Seismic Data
  private seisPlotFrame    plotframe = null; // Seismic Data Plot Frame

  private seisPlotXYFrame  plotXY    = null; // 2D Plot Frame
  private seisPlotXYStruct stPlot[]  = null; // 2D Plot Data Structure
  private int              iPlot     = 0;    // Total number of curves

  private int              iPasses   = 1;    // Number of Passes through signal
  private int              iPoints   = 100;  // Total number of points Default
  private int              iStart    = 0;    // Starting Point Default
  private int              iEnd      = 100;  // Ending Point Default

  private double           dMinimum  = -1.0; // Normalize Minimum Default
  private double           dMaximum  =  1.0; // Normalize Maximum Default

  private String           sTitle    = "";   // Title for plots

  // -- Smoothing Wave Variables

  private static final int _NONE     = seismic.math.seismicMath._NONE;
  private static final int _PULSE    = seismic.math.seismicMath._PULSE;
  private static final int _SQR      = seismic.math.seismicMath._SQR;
  private static final int _SQR_2    = seismic.math.seismicMath._SQR_2;
  private static final int _WAVE     = seismic.math.seismicMath._WAVE;

  private int              iWave     = _NONE;
  private int              iWave2    = _NONE;

  private static final int MINIMUM1  = 0;
  private static final int MAXIMUM1  = 1;

  private static final int PASSES    = 2;
  private static final int POINTS    = 3;

  private static final int START     = 4;
  private static final int END       = 5;

  private static final int MINIMUM   = 6;
  private static final int MAXIMUM   = 7;

  private int              iData     = -1;

  private static final int RAW_DATA  = 0;
  private static final int AVG_DATA  = 1;

  private int              iType     = RAW_DATA;

  // Global Frame Widgets

  // -- Track 1 Minimum & Maximum Limits Widgets

  private JTextField txtMinimum1 = new JTextField();   // Minimum Value
  private JTextField txtMaximum1 = new JTextField();   // Maximum Value

  // Build Smoothing Waveform Widgets

  private JRadioButton rbRaw     = new JRadioButton(); // Set Raw Data
  private JRadioButton rbAvg     = new JRadioButton(); // Set Average Data

  // -- Standard Wave Widgets

  private JTextField txtPasses   = new JTextField();   // Number of Passes
  private JTextField txtPoints   = new JTextField();   // Total Points

  private JRadioButton rbNone    = new JRadioButton(); // NO Selection
  private JRadioButton rbPulse   = new JRadioButton(); // Pulse Wave
  private JRadioButton rbSquare  = new JRadioButton(); // Square Wave
  private JRadioButton rb2Square = new JRadioButton(); // 2 Square Waves

  private JLabel       lblOrder2 = new JLabel();       // 2nd Order of processing

  // -- Constructed Wave from 1st 100 Points of Seismic Wave Widgets

  private JTextField txtStart    = new JTextField();   // Start Point
  private JTextField txtEnd      = new JTextField();   // End Point

  private JTextField txtMinimum  = new JTextField();   // Minimum Normalized Value
  private JTextField txtMaximum  = new JTextField();   // Maximum Normalized Value

  private JRadioButton rbSeismic = new JRadioButton(); // Wave from Seismic Data

  private JLabel       lblOrder1 = new JLabel();       // 1st Order of processing

  // -- Compute Smoothing

  private JButton    btnCompute  = new JButton();      // Compute Button

  /** Constructor seisSmoothingPanel()
   *  <p> This is the Constructor for this class.
   *  @param stCMN    = Global IQSTRAT Data Structure
   *  @param notifier = Observables
   */

  public seisSmoothingPanel( cmnStruct stCMN, Observable notifier )
  {
    try
    {
      this.stCMN    = stCMN;
      this.notifier = notifier;

      jbInit();

      setButtons();
    }
    catch(Exception ex)
    {
      ex.printStackTrace();
    }
  }

  /** METHOD jbInit()
   *  <p> This Method will create the Panel for this class.
   *  @throws Exception
   */

  private void jbInit() throws Exception
  {
    JPanel pnlTop    = new JPanel();
    JPanel pnlFilter = new JPanel();
    JPanel pnlWave   = new JPanel();

    JPanel pnlCenter = new JPanel();
    JPanel pnlTrack1 = new JPanel();

    JPanel pnl       = new JPanel();
    JPanel pnlSmooth = new JPanel();

    ButtonGroup group = new ButtonGroup();

    TitledBorder titledBorderData = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "Set Seismic Data to Filter");
    titledBorderData.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    // Build Other Widgets

    this.setLayout(new BorderLayout());

    // Panel Top

    pnlTop.setLayout(new BorderLayout());

    // Set Data Type for Filter Panel

//    pnlFilter.setLayout(new GridLayout());
    pnlFilter.setBorder(titledBorderData);

    rbRaw.setSelected(true);
    rbRaw.setText("Use Raw Data");
    rbRaw.addActionListener(this);

    rbAvg.setSelected(true);
    rbAvg.setText("Use Filtered Data");
    rbAvg.addActionListener(this);

    // -- Track 1 (Seismic Wave) Panel

    pnlTrack1 = BuildTrack1LimitsPanel();

    // Panel Center

    pnlCenter.setLayout(new BorderLayout());

    // -- Compute Smoothing Panel

    pnlSmooth =  buildConvolveWavePanel();

    // Add Widgets to Frame

    this.add(pnlTop,         BorderLayout.NORTH);
    pnlTop.add(pnlTrack1,    BorderLayout.NORTH);

    pnlTop.add(pnlFilter,    BorderLayout.SOUTH);
    pnlFilter.add(rbRaw,     null);
    pnlFilter.add(rbAvg,     null);

    group.add(rbRaw);
    group.add(rbAvg);

    this.add(pnlCenter,      BorderLayout.CENTER);
    pnlCenter.add(pnlSmooth, BorderLayout.NORTH);
  }

  /** Method buildConvolveWavePanel()
   *  <p> This panel will allow the user to select the method they wish to use
   *      to filter the seismic data to remove excess noise in the signal to
   *      reveal the micro seismic data.
   */

  private JPanel buildConvolveWavePanel()
  {
	JPanel panel          = new JPanel();
    JPanel pnlPulse       = new JPanel();
    JPanel pnlPulseTop    = new JPanel();
    JPanel pnlPulseTop2   = new JPanel();

    JPanel pnlActions     = new JPanel();

    JPanel pnlPasses      = new JPanel();
    JLabel lblPasses      = new JLabel();

    JPanel pnlPoints      = new JPanel();
    JLabel lblPoints      = new JLabel();

    JPanel pnlSeismic     = new JPanel();
    JPanel pnlSeismicTop  = new JPanel();

    JPanel pnlWave        = new JPanel();

    JPanel pnlWidth       = new JPanel();
    JPanel pnlStart       = new JPanel();
    JPanel pnlEnd         = new JPanel();

    JPanel pnlHeight      = new JPanel();
    JPanel pnlMinimum     = new JPanel();
    JPanel pnlMaximum     = new JPanel();

    JPanel pnlBase        = new JPanel();

    ButtonGroup group     = new ButtonGroup();

    TitledBorder titledBorderTitle = new TitledBorder(
        new EtchedBorder(EtchedBorder.RAISED,
                         Color.white,
                         new Color(165, 163, 151)),
        "Convolve Seismic Signal with Wave");
    titledBorderTitle.setTitleFont(new java.awt.Font("Dialog", 1, 13));

    TitledBorder titledBorderWave = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "Create Wave Filter from Seismic Data");
    titledBorderWave.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderPulse = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "Standard Wave Filters");
    titledBorderPulse.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderWidth = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "Wave Point Range");
    titledBorderWidth.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderStart = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "Start");
    titledBorderStart.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderEnd = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "End");
    titledBorderEnd.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderHeight = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "Wave Magnitudes");
    titledBorderHeight.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderMinimum = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "Minimum");
    titledBorderMinimum.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderMaximum = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "Maximum");
    titledBorderMaximum.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    // Main Panel

    panel.setLayout(new BorderLayout());
    panel.setBorder(titledBorderTitle);

    // -- Use standard waveforms Panel

    pnlPulse.setLayout(new BorderLayout());
    pnlPulse.setBorder(titledBorderPulse);

    pnlPulseTop.setLayout(new BorderLayout());

    lblOrder2.setText("");

    pnlPulseTop2.setLayout(new GridLayout(2,2));

    rbNone.setSelected(true);
//    rbNone.setFont(new java.awt.Font("Dialog", 0, 11));
    rbNone.setHorizontalAlignment(SwingConstants.CENTER);
    rbNone.setText("None");
    rbNone.addActionListener(this);

//    rbPulse.setFont(new java.awt.Font("Dialog", 0, 11));
    rbPulse.setHorizontalAlignment(SwingConstants.CENTER);
    rbPulse.setText("Pulse");
    rbPulse.addActionListener(this);

//    rbSquare.setFont(new java.awt.Font("Dialog", 0, 11));
    rbSquare.setHorizontalAlignment(SwingConstants.CENTER);
    rbSquare.setText("Square Wave");
    rbSquare.addActionListener(this);

//    rb2Square.setFont(new java.awt.Font("Dialog", 0, 11));
    rb2Square.setHorizontalAlignment(SwingConstants.CENTER);
    rb2Square.setText("2 Square Wave");
    rb2Square.addActionListener(this);

    group.add(rbNone);
	group.add(rbPulse);
	group.add(rbSquare);
	group.add(rb2Square);
//	group.add(rbSeismic);

    pnlActions.setLayout(new GridLayout());

    pnlPasses.setLayout(new BorderLayout());

    lblPasses.setText("No. of Passes: ");

    txtPasses.setText("" + iPasses);
    txtPasses.setColumns(4);
	txtPasses.setHorizontalAlignment(SwingConstants.TRAILING);
	txtPasses.addFocusListener(new seisSmoothingPanelFocusAdapter(this));

    pnlPoints.setLayout(new BorderLayout());

    lblPoints.setText("  No. of Points: ");

    txtPoints.setText("" + iPoints);
    txtPoints.setColumns(4);
	txtPoints.setHorizontalAlignment(SwingConstants.TRAILING);
	txtPoints.addFocusListener(new seisSmoothingPanelFocusAdapter(this));

    // -- Build wave from seismic data Panel

    pnlSeismic.setLayout(new BorderLayout());
    pnlSeismic.setBorder(titledBorderWave);

    pnlSeismicTop.setLayout(new BorderLayout());

    lblOrder1.setText("");

//    rbSeismic.setFont(new java.awt.Font("Dialog", 0, 11));
    rbSeismic.setHorizontalAlignment(SwingConstants.CENTER);
    rbSeismic.setText("Construct from Seismic Wave");
    rbSeismic.addActionListener(this);

    pnlWave.setLayout(new GridLayout());
//    pnlWave.setBorder(titledBorderWave);

    // .. Set Start & Stop Point Limits Panel

    pnlWidth.setLayout(new GridLayout());
    pnlWidth.setBorder(titledBorderWidth);

    // .... Start Point Panel

    pnlStart.setLayout(new BorderLayout());
    pnlStart.setBorder(titledBorderStart);

    txtStart.setText("" + iStart);
	txtStart.setHorizontalAlignment(SwingConstants.TRAILING);
	txtStart.addFocusListener(new seisSmoothingPanelFocusAdapter(this));

    pnlStart.add(txtStart);

    // .... End Point Panel

    pnlEnd.setLayout(new BorderLayout());
    pnlEnd.setBorder(titledBorderEnd);

    txtEnd.setText("" + iEnd);
	txtEnd.setHorizontalAlignment(SwingConstants.TRAILING);
	txtEnd.addFocusListener(new seisSmoothingPanelFocusAdapter(this));

    pnlEnd.add(txtEnd);

    pnlWidth.add(pnlStart, null);
    pnlWidth.add(pnlEnd,   null);

    // .. Set Normalization Magnitude Limits Panel

    pnlHeight.setLayout(new GridLayout());
    pnlHeight.setBorder(titledBorderHeight);

    // .... Minimum Magnitude Panel

    pnlMinimum.setLayout(new BorderLayout());
    pnlMinimum.setBorder(titledBorderMinimum);

    txtMinimum.setText("" + dMinimum);
	txtMinimum.setHorizontalAlignment(SwingConstants.TRAILING);
	txtMinimum.addFocusListener(new seisSmoothingPanelFocusAdapter(this));

    pnlMinimum.add(txtMinimum);

    // .... Maximum Magnitude Panel

    pnlMaximum.setLayout(new BorderLayout());
    pnlMaximum.setBorder(titledBorderMaximum);

    txtMaximum.setText("" + dMaximum);
	txtMaximum.setHorizontalAlignment(SwingConstants.TRAILING);
	txtMaximum.addFocusListener(new seisSmoothingPanelFocusAdapter(this));

    pnlMaximum.add(txtMaximum);

    pnlHeight.add(pnlMinimum, null);
    pnlHeight.add(pnlMaximum, null);

    // -- Compute Smoothing Panel

    pnlBase.setLayout(new BorderLayout());

    btnCompute.setFont(new java.awt.Font("Dialog", 1, 11));
    btnCompute.setText("Compute");
    btnCompute.addActionListener(this);

    // Attach Widgets to Panel

	panel.add(pnlPulse,           BorderLayout.NORTH);
	pnlPulse.add(pnlPulseTop,     BorderLayout.NORTH);
    pnlPulseTop.add(lblOrder2,    BorderLayout.WEST);

    pnlPulseTop.add(pnlPulseTop2, BorderLayout.CENTER);
    pnlPulseTop2.add(rbNone,      null);
	pnlPulseTop2.add(rbPulse,     null);
	pnlPulseTop2.add(rbSquare,    null);
	pnlPulseTop2.add(rb2Square,   null);

    pnlPulse.add(pnlActions,      BorderLayout.CENTER);
    pnlActions.add(pnlPasses,     null);
    pnlPasses.add(lblPasses,      BorderLayout.CENTER);
    pnlPasses.add(txtPasses,      BorderLayout.EAST);

    pnlActions.add(pnlPoints,     null);
//    pnlPulse.add(pnlPoints,       BorderLayout.CENTER);
    pnlPoints.add(lblPoints,      BorderLayout.CENTER);
    pnlPoints.add(txtPoints,      BorderLayout.EAST);

	panel.add(pnlSeismic,         BorderLayout.CENTER);
	pnlSeismic.add(pnlSeismicTop, BorderLayout.NORTH);
    pnlSeismicTop.add(rbSeismic,  BorderLayout.CENTER);
    pnlSeismicTop.add(lblOrder1,  BorderLayout.WEST);

    pnlSeismic.add(pnlWave,       BorderLayout.CENTER);
    pnlWave.add(pnlWidth,         null);
    pnlWave.add(pnlHeight,        null);

	panel.add(pnlBase,            BorderLayout.SOUTH);
    pnlBase.add(btnCompute,       BorderLayout.CENTER);

	return (panel);
  }

  /** Method BuildTrack1LimitsPanel()
   *  <p> this method will build the Track 1 Data Limits Panel
   */

  private JPanel BuildTrack1LimitsPanel()
  {
    JPanel pnlTrack         = new JPanel();
    JPanel pnlMinimum       = new JPanel();
    JPanel pnlMaximum       = new JPanel();

    TitledBorder titledBorderWaves = new TitledBorder(
        new EtchedBorder(EtchedBorder.RAISED,
                         Color.white,
                         new Color(165, 163, 151)),
        seismic.seisTracksStruct.SEIS_TRACK_DESC[1]+" Plot Track");
    titledBorderWaves.setTitleFont(new java.awt.Font("Dialog", 1, 13));

    TitledBorder titledBorderMinimum = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "Minimum");
    titledBorderMinimum.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderMaximum = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "Maximum");
    titledBorderMaximum.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    // -- Track 1 (Seismic Wave) Panel

    pnlTrack.setLayout(new GridLayout());
    pnlTrack.setBorder(titledBorderWaves);

    pnlMinimum.setLayout(new BorderLayout());
    pnlMinimum.setBorder(titledBorderMinimum);
    pnlMinimum.add(txtMinimum1);

    txtMinimum1.setText("");
	txtMinimum1.setHorizontalAlignment(SwingConstants.TRAILING);
	txtMinimum1.addFocusListener(new seisSmoothingPanelFocusAdapter(this));

    pnlMaximum.setLayout(new BorderLayout());
    pnlMaximum.setBorder(titledBorderMaximum);
    pnlMaximum.add(txtMaximum1);

    txtMaximum1.setText("");
	txtMaximum1.setHorizontalAlignment(SwingConstants.TRAILING);
	txtMaximum1.addFocusListener(new seisSmoothingPanelFocusAdapter(this));

    pnlTrack.add(pnlMinimum, null);
    pnlTrack.add(pnlMaximum, null);

    return (pnlTrack);
  }

  /* ===================================================================== *
   * ----------------------------- CLOSE --------------------------------- *
   * ===================================================================== */

  /** Method close()
   * <p> This method will close all dialogs that are opened by this class.
   */

  public void close()
  {
    // Input Variables

    stCMN     = null; // Global IQSTRAT Data Structure
    notifier  = null; // Observables Object

    // Global Variables

//    if (stData != null)
//      stData.delete();
    stData    = null; // Seismic Data Structure

    if (stAvg != null)
      stAvg.delete();
    stAvg     = null; // Seismic Data Structure

//    if (stMag != null)
//      stMag.delete();
    stMag     = null; // Seismic Data Structure

    plotframe = null; // Seismic Data Plot Frame

    closeXYPlot();

    // Global Frame Widgets

    // -- Track 1 Minimum & Maximum Limits Widgets

    txtMinimum1 = null; // Minimum Value
    txtMaximum1 = null; // Maximum Value

    // Build Smoothing Waveform Widgets

    rbRaw       = null;
    rbAvg       = null;

    // -- Standard Wave Widgets

    txtPasses   = null; // Number of Passes
    txtPoints   = null; // Total Points

    rbNone      = null; // None
    rbPulse     = null; // Pulse Wave
    rbSquare    = null; // Square Wave
    rb2Square   = null; // 2 Square Waves

    // -- Constructed Wave from 1st 100 Points of Seismic Wave Widgets

    txtStart    = null; // Start Point
    txtEnd      = null; // End Point

    txtMinimum  = null; // Minimum Normalized Value
    txtMaximum  = null; // Maximum Normalized Value

    rbSeismic   = null; // Wave from Seismic Data

    // -- Compute Smoothing

    btnCompute  = null; // Compute Button
  }

  /** Method closeXYPlot()
   * <p> This method will close the XY Plot
   */

  private void closeXYPlot()
  {
    if (plotXY != null)
      plotXY.close();
    plotXY    = null; // XY Plot Frame

    if (iPlot > 0)
    {
	  for (int i=0; i<iPlot; i++)
	  {
		if (stPlot[i] != null)
		  stPlot[i].delete();
		stPlot[i]  = null;
	  }

	  stPlot  = null;
	}
  }

  /* ===================================================================== *
   * -------------------------- GET METHODS ------------------------------ *
   * ===================================================================== */

  /** Method getData()
   *  <p> This method will retrieve the Averaged Data Set
   *  @return stAvg = the averaged seismic data structure
   */

  public seisStruct getData() { return (stAvg); }

  /** Method getTitle()
   *  <p> This method will retrieve the title from this dialog
   */

  public String getTitle() {  return (sTitle); }

  /* ===================================================================== *
   * -------------------------- SET METHODS ------------------------------ *
   * ===================================================================== */

  /** Method setData(seisStruct st)
   * <p> This method will set the Seismic Data Structure
   * @param st - Seismic Data Structure
   */

  public void setData(seisStruct st)
  {
	this.stData = st;
	plotXY();
  }

  /** Method setMagData(seisStruct st)
   * <p> This method will set the Seismic Data Structure
   * @param st - Seismic Data Structure
   */

  public void setMagData(seisStruct st)
  {
	this.stMag = st;
	plotXY();
  }

  /** Method setPlot()
   *  <p> This method will pass the plotframe pointer
   *  @plotframe = the plot frame pointer
   */

  public void setPlot(seisPlotFrame plotframe)
  {
	this.plotframe = plotframe;
  }

  /** Method setTitle()
   *  <p> This method will retrieve the title from this dialog
   *  @param sTitle = The title from the frequency analysis
   */

  public void setTitle(String sTitle)
  {
	this.sTitle = new String(sTitle);
  }

  /** Method setButtons()
   *  <p> This method will enable or disable widgets on this frame
   */

  private void setButtons()
  {
	rbAvg.setEnabled(false);

	if (stAvg != null)
	  rbAvg.setEnabled(true);

	btnCompute.setEnabled(false);
	if ((iWave != _NONE) || (iWave2 != _NONE))
	  btnCompute.setEnabled(true);

	if (((rbPulse.isSelected()) || (rbSquare.isSelected()) ||
	     (rb2Square.isSelected())) && (rbSeismic.isSelected()))
	{
      lblOrder1.setText("1st");
      lblOrder2.setText("2nd");
	}
	else
	{
      lblOrder1.setText("");
      lblOrder2.setText("");
	}
  }

  /* ===================================================================== *
   * -------------------------- PLOT METHOD ------------------------------ *
   * ===================================================================== */

  /** Method plotXY()
   *  <p> This method will plot the 1st few points of the raw data
   */

  private void plotXY()
  {
	int              iData    = 100;
	double           data[]   = null;
	int              iCount   = 0;
	double           dMin     = 0.0;
	double           dMax     = 0.0;

    closeXYPlot();

    stPlot = new seisPlotXYStruct[3];
    iPlot  = 0;

    if (stData != null)
    {
      data = new double[iData];
      for (int i=0; i<stData.iRows; i++)
      {
		if (iCount < iData)
		{
		  data[iCount] = stData.data[i];

		  if (data[iCount] < dMin) { dMin = data[iCount]; }
		  if (data[iCount] > dMax) { dMax = data[iCount]; }

		  iCount++;
		}
	  }

      stPlot[iPlot] = loadData(data);
      iPlot++;
	}

    if (stAvg != null)
    {
	  iCount = 0;
      data   = new double[iData];

      for (int i=0; i<stAvg.iRows; i++)
      {
		if (iCount < iData)
		{
		  data[iCount] = stAvg.data[i];

		  if (data[iCount] < dMin) { dMin = data[iCount]; }
		  if (data[iCount] > dMax) { dMax = data[iCount]; }

		  iCount++;
		}
	  }

      stPlot[iPlot] = loadData(data);
      iPlot++;
	}

    if (stMag != null)
    {
	  iCount = 0;
      data   = new double[iData];

      for (int i=0; i<stMag.iRows; i++)
      {
		if (iCount < iData)
		{
		  data[iCount] = stMag.data[i];

		  if (data[iCount] < dMin) { dMin = data[iCount]; }
		  if (data[iCount] > dMax) { dMax = data[iCount]; }

		  iCount++;
		}
	  }

      stPlot[iPlot] = loadData(data);
      iPlot++;
	}

    if (iPlot > 1)
    {
	  if (stPlot[1].dYMinimum < stPlot[0].dYMinimum)
	    stPlot[0].dYMinimum = stPlot[1].dYMinimum;

	  if (stPlot[1].dYMaximum > stPlot[0].dYMaximum)
	    stPlot[0].dYMaximum = stPlot[1].dYMaximum;

	  stPlot[0].dYMaximum = (double) ((int)(stPlot[0].dYMaximum) + 1);
	  stPlot[0].dYMinimum = (double) ((int)(stPlot[0].dYMinimum) - 1);
	}

    if (stPlot != null)
    {
      if (stPlot[0].iRows > 0)
      {
        plotXY  = new seisPlotXYFrame( iPlot, stPlot );
        plotXY.setTitle1("First 100 points of Seismic Signal");
        if (iPlot == 2)
        {
		  plotXY.setTitle2("Red Curve - Raw Data; Green Curve - Averaged Data");
        }
        plotXY.setTitle3(sTitle);
	  }
    }
  }

  /** Method loadData()
   *  <p> This method will load the plot data structure
   */

  private static seisPlotXYStruct loadData(double d[])
  {
	int              i     = 0;
    seisPlotXYStruct st    = new seisPlotXYStruct();
    int              iMin  = 0;
    int              iMax  = 0;
    double           dMin  = 0.0;
    double           dMax  = 0.0;
    double           dTemp = 0.0;

	if (d != null)
	{
	  st.sTitle1       = new String("First 100 Points of Seismic Signal Data");

	  if (d.length > 0)
	  {
        st.iXLog       = 0;                    // Plot Log
        st.iXCycles    = 1;                    // Number of Cycles
        st.dXMaximum   = 100.0;                // Maximum X-Axis
        st.dXMinimum   = 0.0;                  // Minimum X-Axis
        st.dXIncrement = 10.0;                 // Increment along the X-Axis
        st.sXMnemonic  = new String("No");     // X-Axis Mnemonic
        st.sXAxis      = new String("Number (1000 pts = 1 second)"); // X-Axis Label.

        for (i=0; i<d.length; i++)
        {
	      if (i==0)
	      {
	    	dMin = d[i];
	    	dMax = d[i];
	      }

	      if (d[i] < dMin) { dMin = d[i]; }
	      if (d[i] > dMax) { dMax = d[i]; }
	    }

        if (dMin == dMax)
        {
	      dMax = dMin + 1.0;
	    }


        st.iYLog       = 0;                       // Plot Log
        st.iYCycles    = 1;                       // Number of Cycles
        st.dYMaximum   = dMax;                    // Maximum X-Axis
        st.dYMinimum   = dMin;                    // Minimum X-Axis
        st.dYIncrement = (dMax - dMin) / 10.0;
        st.sYMnemonic  = new String("MAG");       // X-Axis Mnemonic
        st.sYAxis      = new String("Magnitude"); // X-Axis Label.

        // Plot Data.

        st.iRows     = d.length;             // Number of Rows
        st.depths    = new double[st.iRows]; // Array of Depths
        st.dX        = new double[st.iRows]; // Array of Horizontal Axis Data
        st.dY        = new double[st.iRows]; // Array of Vertical Axis Data
        st.dZ        = new double[st.iRows]; // Array of Other Data
        st.sZ        = new String[st.iRows]; // Data descriptor array for filtering
        st.stSymbols = new plotSymbolStruct[st.iRows]; // Array of Symbol & colors

        for (i=0; i<d.length; i++)
        {
          st.depths[i]    = 0.0;
          st.dX[i]        = (double) i;
          st.dY[i]        = d[i];
          st.dZ[i]        = 0.0;
          st.sZ[i]        = new String("");
          st.stSymbols[i] = plot.plotSymbol.getPlotSymbolStructure(
                                  plot.plotSymbolStruct.FILLED,
                                  plot.plotSymbolStruct.OVAL,
                                  0, 0, 0, "NONE");
//System.out.println(i+" "+st.dX[i]+" "+st.dY[i]);
	    }
	  }
	}

	return (st);
  }

  /* ===================================================================== *
   * ------------------------- COMPUTE METHODS --------------------------- *
   * ===================================================================== */

  /** Method Compute()
   *  <p> This method will create the wave form and filter the data
   */

  private void Compute()
  {
	int iPass = 1;

	iPass = cmn.cmnString.stringToInt(txtPasses.getText());

	if (iWave2 != _NONE)
	{
	  ComputeCreated();
	}

	if (iWave != _NONE)
	{
	  for (int i=0; i<iPass; i++)
	  {
	    if (iWave2 == _NONE)
	    {
	  	  if ((i==0) && (iType == RAW_DATA))
	        ComputeStandard(stData);
	      else
	        ComputeStandard(stAvg);
	    }
	    else
	      ComputeStandard(stAvg);
	  }
	}

    txtMinimum1.setText("" + stAvg.dMinimum);
    txtMaximum1.setText("" + stAvg.dMaximum);

    plotframe.setData(_SEISMIC_AVG, stAvg); // Average Seismic Data structure
    plotframe.refresh();

//    stMag  = seismic.math.seismicMath.envelope(stAvg);
//    plotframe.setData(_SEISMIC_MAG, stMag); // Average Seismic Data structure

	plotXY();

    setButtons();

    if (notifier != null)
      notifier.notifyObservers(new String("Average Complete"));
  }

  /** Method ComputeStandard()
   *  <p> This method will use standard wave form and filter the data
   *  @param st = seismic data structure
   */

  private void ComputeStandard(seisStruct st)
  {
	int NPts = 0;

	if (st != null)
	{
      NPts  = cmn.cmnString.stringToInt(txtPoints.getText());
      stAvg = seismic.math.seismicMath.movingAverage( st, iWave,  NPts );
	}
  }

  /** Method ComputeCreated()
   *  <p> This method will use created wave form and filter the data
   */

  private void ComputeCreated()
  {
	int    NPts   = 0;
	int    NStart = 0;
	int    NEnd   = 0;
	double dMin   = 0.0;
	double dMax   = 0.0;
	double data[] = null;
	seisStruct st = null;

	if (iType == RAW_DATA)
	  st = stData;
	else
	  st = stAvg;

    NStart = cmn.cmnString.stringToInt(txtStart.getText());
    NEnd   = cmn.cmnString.stringToInt(txtEnd.getText());
    dMin   = cmn.cmnString.stringToDouble(txtMinimum.getText());
    dMax   = cmn.cmnString.stringToDouble(txtMaximum.getText());

    data   = seismic.math.seismicMath.loadData( NStart, NEnd, dMin, dMax, st);

    NPts   = data.length;
    stAvg  = seismic.math.seismicMath.movingAverage( st, NPts, data );
  }

  /** Method WaitCompute()
   * <p> This method will display the Wait Animation for computing the non
   *     linear component
   */

  private void WaitCompute()
  {
    java.net.URL url = null;

    // Get the URL Path to display the Wait Animation

    try
    {
      url = new java.net.URL(cmn.cmnStruct.IQSTRAT_BASE + "/");
    }
    catch(Exception ex)
    {
      System.out.println("Wrong Image Directory" + ex);
    }

    // Start the Wait Animation Dialog

    final utilWaitAnimation pDialog = new utilWaitAnimation(url);
    pDialog.startAnimation();

    final SwingWorker worker = new SwingWorker()
    {
      public Object construct()
      {
        Compute();

        return "OK";
      }

      //Runs on the event-dispatching thread.

      public void finished() { pDialog.stopAnimation(); }
    };

    // Start the Wait Animation Thread

    worker.start();  //required for SwingWorker
  }

  /* ===================================================================== *
   * ---------------------------- ACTIONS -------------------------------- *
   * ===================================================================== */

  /** METHOD actionPerformed()
   *  <p> This Method will handle all the actions within the Frame.
   *  @param event - Action Event
   */

  public void actionPerformed(ActionEvent event)
  {
	if (event.getSource() == rbRaw)     { iType = RAW_DATA; }  // Raw Data
	if (event.getSource() == rbAvg)     { iType = AVG_DATA; }  // Filtered Data

	if (event.getSource() == rbNone)    { iWave = _NONE; }  // None
    if (event.getSource() == rbPulse)   { iWave = _PULSE; } // Pulse Wave
    if (event.getSource() == rbSquare)  { iWave = _SQR; }   // Square Wave
    if (event.getSource() == rb2Square) { iWave = _SQR_2; } // 2 Square Waves
    if (event.getSource() == rbSeismic)  // Wave from Seismic Data
    {
	  if (rbSeismic.isSelected())
	    iWave2 = _WAVE;
	  else
	    iWave2 = _NONE;
	}

    setButtons();

    if (event.getSource() == btnCompute) { WaitCompute(); }
  }

  /** METHOD focusLost()
   *  <p> This Method will handle the lost focus event for the text fields.
   *  @param e      = Focus Lost Event
   */

  protected void focusLost(FocusEvent e)
  {
	focusLostTrack1(e);
	focusLostWave(e);
  }

  /** METHOD focusLostTrack1()
   *  <p> This Method will handle the lost focus event for the text fields.
   *  @param e      = Focus Lost Event
   */

  protected void focusLostTrack1(FocusEvent e)
  {
    int     iNumeric = 0;
    int     len      = 0;
    int     iField   = -1;
    boolean bVal     = true;
    String  sMessage = new String("");
    String  sTemp    = new String("");
    int     id       = 0;
    double  d        = 0.0;
    double  d1       = 0.0;

    if (e.getSource() == txtMinimum1)
    {
      iData    = MINIMUM1;
      iNumeric = 1;
      sTemp    = new String( txtMinimum1.getText().trim());
      sMessage = new String("Minimum Value is a Numeric Field");
    }

    if (e.getSource() == txtMaximum1)
    {
      iData    = MAXIMUM1;
      iNumeric = 1;
      sTemp    = new String( txtMaximum1.getText().trim());
      sMessage = new String("Maximum Value is a Numeric Field");
    }

    if (iNumeric == 1)
    {
      if (!cmn.cmnString.isNumeric(sTemp))
      {
        JOptionPane.showMessageDialog((Component) null,
                                       sMessage,
                                       "ERROR",
                                       JOptionPane.ERROR_MESSAGE);

        switch (iData)
        {
          case MINIMUM1:
            txtMinimum1.setText(""+stAvg.dMinimum);
            break;

          case MAXIMUM1:
            txtMaximum1.setText(""+stAvg.dMaximum);
            break;
	    }
      }
      else
      {
        switch (iData)
        {
          case MINIMUM1:
          case MAXIMUM1:
            d  = cmn.cmnString.stringToDouble(txtMinimum1.getText());
            d1 = cmn.cmnString.stringToDouble(txtMaximum1.getText());
            plotframe.setLimits( _SEISMIC_AVG, d, d1 );
            break;
	    }
      }
    }
  }

  /** METHOD focusLostWave()
   *  <p> This Method will handle the lost focus event for the text fields.
   *  @param e      = Focus Lost Event
   */

  protected void focusLostWave(FocusEvent e)
  {
    int     iNumeric = 0;
    int     len      = 0;
    int     iField   = -1;
    boolean bVal     = true;
    String  sMessage = new String("");
    String  sTemp    = new String("");
    int     id       = 0;
    double  d        = 0.0;
    double  d1       = 0.0;

    if (e.getSource() == txtPasses)
    {
      iData    = PASSES;
      iNumeric = 1;
      sTemp    = new String( txtPasses.getText().trim());
      sMessage = new String("Number of Passes Value is a Numeric Field");
    }

    if (e.getSource() == txtPoints)
    {
      iData    = POINTS;
      iNumeric = 1;
      sTemp    = new String( txtPoints.getText().trim());
      sMessage = new String("Points Value is a Numeric Field");
    }

    if (e.getSource() == txtStart)
    {
      iData    = START;
      iNumeric = 1;
      sTemp    = new String( txtStart.getText().trim());
      sMessage = new String("Start Value is a Numeric Field");
    }

    if (e.getSource() == txtEnd)
    {
      iData    = END;
      iNumeric = 1;
      sTemp    = new String( txtEnd.getText().trim());
      sMessage = new String("End Value is a Numeric Field");
    }

    if (e.getSource() == txtMinimum)
    {
      iData    = MINIMUM;
      iNumeric = 1;
      sTemp    = new String( txtMinimum.getText().trim());
      sMessage = new String("Minimum Value is a Numeric Field");
    }

    if (e.getSource() == txtMaximum)
    {
      iData    = MAXIMUM;
      iNumeric = 1;
      sTemp    = new String( txtMaximum.getText().trim());
      sMessage = new String("Maximum Value is a Numeric Field");
    }

    if (iNumeric == 1)
    {
      if (!cmn.cmnString.isNumeric(sTemp))
      {
        JOptionPane.showMessageDialog((Component) null,
                                       sMessage,
                                       "ERROR",
                                       JOptionPane.ERROR_MESSAGE);

        switch (iData)
        {
		  case PASSES:
		    txtPasses.setText(""+iPasses);
		    break;

		  case POINTS:
		    txtPoints.setText(""+iPoints);
		    break;

		  case START:
		    txtStart.setText(""+iStart);
		    break;

		  case END:
		    txtEnd.setText(""+iEnd);
		    break;

          case MINIMUM:
            txtMinimum.setText(""+dMinimum);
            break;

          case MAXIMUM:
            txtMaximum.setText(""+dMaximum);
            break;
	    }
      }
    }
  }
}

/** CLASS seisSmoothingPanelFocusAdapter()
  *  <p> This Class will handle Actions when focus is lost on a textfield
  */

class seisSmoothingPanelFocusAdapter extends java.awt.event.FocusAdapter
{
  seisSmoothingPanel adaptee;

  seisSmoothingPanelFocusAdapter(seisSmoothingPanel adaptee)
  {
    this.adaptee = adaptee;
  }

  public void focusLost(FocusEvent e) { adaptee.focusLost(e); }
}

/*
 *  @version 1.1 03/24/2015
 *  @author  John Victorine
 */

