/*
 * @seisPlotTrack.java Version 1.1 03/04/2015
 *
 * Copyright (c) 2015 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package seismic.plot;

import java.awt.*;
import java.awt.geom.*;

import java.sql.Timestamp;

import seismic.seisStruct;

/** CLASS seisPlotTrack
 *  <p> This Class will create the seismic track.
 *
 *  @version 1.1 03/04/2015
 *  @author  John R. Victorine
 */

public class seisPlotTrack extends Canvas
{
  // Input Variables

  private int       iScale  = -1;     // Time Scale minutes / inch
  private Timestamp tsStart = null;   // Starting Date
  private Timestamp tsEnd   = null;   // Ending Date

  // Global Variables

  public static final Color cRED     = new Color( 180,   0,   0 );
  public static final Color cGREEN   = new Color(   0, 180,   0 );
  public static final Color cBLUE    = new Color(   0,   0, 180 );
  public static final Color cCYAN    = new Color(   0, 180, 180 );
  public static final Color cMAGENTA = new Color( 180,   0, 180 );
  public static final Color cYELLOW  = new Color( 180, 180,   0 );
  public static final Color cORANGE  = new Color( 204, 102,   0 );
  public static final Color cBROWN   = new Color( 102, 102,   0 );

  public static final Color COLORS[] =  // Color for Log Curves
      { cRED, cGREEN, cBLUE, cBROWN, cMAGENTA, cORANGE, cYELLOW, cCYAN };

  private seisStruct st  = null; // seismic Data Structure

  // Plot Variables

  private int iWidth      = 100;  // Width of Plot
  private int iHeight     = 100;  // Height of Plot with the Titles
  private int iLogHeight  = 100;  // Height of Plot without the Titles
  private int iIncrementY = 100;  // Increment Height

  public static final int PLOT_TITLES = seismic.seisTracksStruct.PLOT_TITLES;
  public static final int LABELSTART  = seismic.seisTracksStruct.LABELSTART;

  /** CONSTRUCTOR seisPlotTrack()
   *  <p> This is the constructor for this class.
   *  @param tsStart  = The starting date
   *  @param tsEnd    = The ending date
   *  @param iScale   = The scale
   */

  public seisPlotTrack( Timestamp tsStart,
                        Timestamp tsEnd,
                        int       iScale )
  {
    setPlotHeight(iScale, tsEnd, tsStart);

    this.setBackground(Color.white);
  }

  /** Method close()
   * <p> This method will set object to null to force JAVA to reallocate memory
   */

  public void close()
  {
	st      = null;
	tsStart = null;
	tsEnd   = null;
  }

  /* =============================================================== *
   * ------------------------- GET METHODS ------------------------- *
   * =============================================================== */

  /** Method getPlotWidth()
   * <p> This method will return the Plot Width
   * @return iWidth = The Plot Width
   */

  public int getPlotWidth()  { return (iWidth); }

  /** Method getPlotHeight()
   * <p> This method will return the Plot Height
   * @return iHeight = The Plot Height
   */

  public int getPlotHeight() { return (iHeight); }

  /* =============================================================== *
   * ------------------------- SET METHODS ------------------------- *
   * =============================================================== */

  /** Method setPlotHeight()
   *  <p> This method will set the Plot Height.
   * @param iScale    = The Time Track Scale identifier
   * @param tsMaximum = The Maximum Date
   * @param tsMinimum = The Minimum Date
   */

  public void setPlotHeight(int iScale, Timestamp tsMaximum, Timestamp tsMinimum)
  {
    this.tsStart = new Timestamp(tsMinimum.getTime());
    this.tsEnd   = new Timestamp(tsMaximum.getTime());
    this.iScale  = iScale;

    long lstart  = tsMinimum.getTime();
    long lend    = tsMaximum.getTime();
    long diff    = lend-lstart;

    iLogHeight   = (int) ((100 * (double) diff) /
                     seismic.seisTracksStruct.SCALE[iScale]);

    iHeight      = PLOT_TITLES + iLogHeight;
  }

  /** Method setData( seisStruct st )
   * <p> This method will set the seismic Data Structure
   * @param st - seismic Data Structure
   */

  public void setLimits(double dMinimum, double dMaximum)
  {
	st.dMinimum = dMinimum;
	st.dMaximum = dMaximum;
  }

  /** Method setData( seisStruct st )
   * <p> This method will set the seismic Data Structure
   * @param st - seismic Data Structure
   */

  public void setData(seisStruct st) { this.st = st; }

  /* =============================================================== *
   * ------------------------ DRAW METHODS ------------------------- *
   * =============================================================== */

  /** Method drawGrid( Graphics g, int iSEIS, int iStartTrack, int iWidthTrack )
   * <p> This method will draw the depth scale
   * @param g           = Graphics Parameter.
   * @param iSEIS        = The LAS Track to draw
   * @param iStartTrack = The starting pixel for the LAS Track
   * @param iWidthTrack = The width of LAS Track in pixels
   */

  public void drawGrid( Graphics g, int iSEIS, int iStartTrack, int iWidthTrack )
  {
    int    i=0;
    int    j=0;
    int    jXinc  = 0;
    int    iLog   = 0;
    int    iCycle = 1;
    double dXinc  = 0.0;
    double dTemp  = 0.0;
    int    iIncrement  = iWidthTrack/4;
    int    iEndTrack   = iStartTrack+iWidthTrack;
//    int    iTrack      = iSEIS - seismic.seisTracksStruct.LAS_START;
    int    iTrack      = iSEIS;
    int    iCurve      = seismic.seisTracksStruct.SEIS_TRACK[iTrack][0];
    int    iValue      = 0;
    int    iY1         = 0;
    int    iY2         = 0;

    double dMinimum = -100.0;
    double dMaximum =  100.0;

    if (st != null)
    {
	  dMaximum = st.dMaximum;
	  dMinimum = st.dMinimum;
	}

    g.setColor(Color.black);

    Font fsb = new Font("Serif", Font.BOLD, 12);
    FontMetrics fsbm = g.getFontMetrics(fsb);
    g.setFont( fsb );

    for (i=0; i<=iLogHeight; i+=iIncrementY)
    {
      iY1 = i+PLOT_TITLES;

      jXinc = iIncrementY / 5;
      for (j=0; j<5; j++)
      {
        iY2 = i+(j*jXinc)+PLOT_TITLES;

        if ((iY2 >= PLOT_TITLES)  && (iY2 < iHeight))
        {
          g.setColor(Color.lightGray);
          g.drawLine(iStartTrack, iY2, iEndTrack, iY2);
        }
      }

      g.setColor(Color.gray);
      g.drawLine(iStartTrack, iY1, iEndTrack, iY1);
    }

    g.setColor(Color.gray);
    g.drawLine(iEndTrack, LABELSTART, iEndTrack, PLOT_TITLES+iLogHeight);

    for (i=0; i<=iWidthTrack; i+=iIncrement)
    {
      g.setColor(Color.gray);
      if ((i==0) || (i==iWidthTrack))
      {
        g.drawLine(i+iStartTrack, LABELSTART,
                   i+iStartTrack, PLOT_TITLES+iLogHeight);
      }
      else
      {
        g.drawLine(i+iStartTrack, PLOT_TITLES,
                   i+iStartTrack, PLOT_TITLES+iLogHeight);
      }

//        g.setColor(Color.lightGray);
      g.setColor(new Color(230, 230, 230));
      for (j=1; j<5; j++)
      {
        dTemp = (double) iIncrement / 5.0;
        jXinc = i + iStartTrack + j * (int) dTemp;

        if (jXinc <= iEndTrack)
        {
          g.drawLine(jXinc, PLOT_TITLES,
                     jXinc, PLOT_TITLES+iLogHeight);
        }
      }
    }

    g.setColor(Color.black);
    g.drawLine(iStartTrack, LABELSTART,  iEndTrack,   LABELSTART);
    g.drawLine(iStartTrack, PLOT_TITLES, iEndTrack,   PLOT_TITLES);
    g.drawLine(iStartTrack, LABELSTART,  iStartTrack, PLOT_TITLES+iLogHeight); //iHeight);
    g.drawLine(iEndTrack,   LABELSTART,  iEndTrack,   PLOT_TITLES+iLogHeight); //iHeight);
  }

   /** Method drawTrackLabels( Graphics g,
    *                          int iSEIS,
    *                          int iStartTrack,
    *                          int iWidthTrack )
    *  <p> This method will draw the labels on the LAS Plot.
    *   @param g           = Graphics Parameter.
    *   @param iSEIS        = The LAS Track to draw
    *   @param iStartTrack = The starting pixel for the LAS Track
    *   @param iWidthTrack = The width of LAS Track in pixels
    */

   public void drawTrackLabels( Graphics g,
                                int iSEIS,
                                int iStartTrack,
                                int iWidthTrack )
   {
     int    i=0;
     int    iCurve      = -1;
     int    iContinue   = 0;
     int    iLine       = -1;
     int    length      = 0;
     String sLabel      = "";
     String sTemp       = "";
     String sUnits      = "";
     int    iEndTrack   = iStartTrack+iWidthTrack;
//     int    iTrack      = iSEIS - seismic.seisTracksStruct.LAS_START;
     int    iTrack      = iSEIS;

     double dMinimum    = -100.0;
     double dMaximum    =  100.0;

     if (st != null)
     {
	   dMaximum = st.dMaximum;
	   dMinimum = st.dMinimum;
	 }

     iLine++;
     for (i=0; i<5; i++)
     {
       iCurve = seismic.seisTracksStruct.SEIS_TRACK[iTrack][i];

       iContinue = 1;
//       if (stLASFileData != null)
//       {
//         if (!stData.checkData(iCurve))
//         {
//           iContinue = 0;
//         }
//         else
//         {
//           iLine++;
//         }
//       }

     if ((iCurve > -1) && (iContinue == 1))
     {
       sLabel = new String(seismic.seisStandardTools.CURVES[iCurve][1]);
       length = sLabel.length();

        Font fsb = new Font("Serif", Font.BOLD, 10);
        g.setFont( fsb );

        if (iLine > -1)
        {
          g.setColor(COLORS[i]);
          g.drawString( sLabel,
                        (iStartTrack+iWidthTrack/2 - 2*length),
                        LABELSTART + (iLine+2)*14);

          g.drawString( "" + dMinimum,
                        iStartTrack+1,
                        LABELSTART + (iLine+2)*14);

          sTemp = new String("" + dMaximum);
          length = sTemp.length();
          g.drawString( "" + dMaximum,
                        iEndTrack-length*5,
                        LABELSTART + (iLine+2)*14);
        }
      }
	}

    g.setColor(Color.black);
    g.drawLine(iStartTrack, LABELSTART, iEndTrack,   LABELSTART);

    sLabel = new String( seismic.seisTracksStruct.SEIS_TRACK_DESC[iTrack] );
    length = sLabel.length();
    g.drawString(sLabel, iStartTrack+iWidthTrack/2-(9*length)/4, LABELSTART+14);
  }

  /** Method drawTrackCurves( Graphics g,
   *                          int iSEIS,
   *                          int iStartTrack,
   *                          int iWidthTrack )
   *  <p> This method will draw the curves on the LAS Plot.
   *   @param g           = Graphics Parameter.
   *   @param iSEIS        = The LAS Track to draw
   *   @param iStartTrack = The starting pixel for the LAS Track
   *   @param iWidthTrack = The width of LAS Track in pixels
   */

  public void drawTrackCurves( Graphics g,
                               int iSEIS,
                               int iStartTrack,
                               int iWidthTrack )
  {
    int    iColumns    = 0;
    int    iPlot       = 0;
    int    iX1         = 0;
    int    iX2         = 0;
    int    iY1         = 0;
    int    iY2         = 0;
    int    iLog        = 0;
    int    iColor      = 0;
    double dRatio      = 0.0;
    String sUnits      = "";
    double dLog[]      = null;
    double dStart      = 0.0;
    double dEnd        = 0.0;
    double data        = 0.0;
    double data1       = 0.0;
    int    iEndTrack   = iStartTrack+iWidthTrack;
//    int    iTrack      = iSEIS - seismic.seisTracksStruct.LAS_START;
    int    iTrack      = iSEIS;
    int    iCurve      = seismic.seisTracksStruct.SEIS_TRACK[iTrack][0];

    double dMinimum = -100.0;
    double dMaximum =  100.0;

    if (st != null)
    {
	  dMaximum = st.dMaximum;
	  dMinimum = st.dMinimum;
	}

    if (st != null)
    {
      for (int i=0; i<5; i++)
      {
        dLog   = null;
        iCurve = seismic.seisTracksStruct.SEIS_TRACK[iTrack][i];

        if (iCurve > -1)
        {
          iColor   = i;
          iColumns = st.iRows-1;

          dLog = st.data; //st.getData(iCurve);

          if (dLog != null)
          {
            for (int j=0; j<iColumns-1; j++)
            {
              if (st != null)
              {
                long lstart = tsStart.getTime();
                long lend   = tsEnd.getTime();

				long l1     = st.ts[j].getTime();
				long l2     = st.ts[j+1].getTime();

                data        = dLog[j];
                data1       = dLog[j+1];

                dRatio = (double) iLogHeight * (l1 - lstart) / (lend - lstart);
                iY1 = PLOT_TITLES + (int) dRatio;

                dRatio = (double) iLogHeight * (l2 - lstart) / (lend - lstart);
                iY2 = PLOT_TITLES + (int) dRatio;

                iPlot = 0;
                if ((iY1 > PLOT_TITLES) && (iY2 > PLOT_TITLES) &&
                    (iY1 < iHeight)     && (iY2 < iHeight))
                {
                  iPlot = 1;
                }

                dRatio = (double) iWidthTrack *
                      (data - dMinimum) / (dMaximum - dMinimum);
                iX1 = iStartTrack + (int) dRatio;

                dRatio = (double) iWidthTrack *
                      (data1 - dMinimum) / (dMaximum - dMinimum);
                iX2 = iStartTrack + (int) dRatio;

                if ((iX1 >= iStartTrack) && (iX2 >= iStartTrack) &&
                    (iX1 <= iEndTrack)   && (iX2 <= iEndTrack))
                {
                  g.setColor(COLORS[iColor]);
                  if (iPlot == 1)
                    g.drawLine(iX1, iY1, iX2, iY2);
			    }
			  }
			}
		  }
		}
	  }
	}
  }

  /** Method draw( Graphics g, int iSEIS, int iStartTrack, int iWidthTrack )
   * <p> This method will create a LAS Plot Track
   * @param g           = Graphics Parameter.
   * @param iSEIS        = The LAS Track to draw
   * @param iStartTrack = The starting pixel for the LAS Track
   * @param iWidthTrack = The width of LAS Track in pixels
   */

  public void draw( Graphics g, int iSEIS, int iStartTrack, int iWidthTrack )
  {
    int iColor = seismic.seisTracksStruct._SRC_SEISMIC;

    if ((iSEIS == seismic.seisTracksStruct._SEISMIC) ||
        (iSEIS == seismic.seisTracksStruct._SEISMIC_AVG) ||
        (iSEIS == seismic.seisTracksStruct._SEISMIC_MAG))
    {
      g.setColor( new Color( seismic.seisTracksStruct.COLORS[iColor][0],
                             seismic.seisTracksStruct.COLORS[iColor][1],
                             seismic.seisTracksStruct.COLORS[iColor][2] ) );
      g.fillRect( iStartTrack, LABELSTART, iWidthTrack, 20 );

      drawGrid( g, iSEIS, iStartTrack, iWidthTrack );
      drawTrackLabels( g, iSEIS, iStartTrack, iWidthTrack );
      drawTrackCurves( g, iSEIS, iStartTrack, iWidthTrack );
	}
  }

  /** Method paint( Graphics g )
   * <p> This method will paint Porosity & Permeability Plot
   * @param g = Graphics Parameter.
   */

  public void paint( Graphics g )
  {
    g.setColor(Color.white);
    g.fillRect(0, 0, iWidth, iHeight);
  }
}

/*
 *  @version 1.1 03/04/2015
 *  @author  John Victorine
 */
